/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSBOXED_H
#define PAPYRUSBOXED_H

#include <papyrus/group.h>
#include <papyrus/rectangle.h>

/**
 * \page guide_boxed Boxed Groups
 *
 * Boxed groups, as the name implies are simply a group that in addition to maintaining
 * a group, also maintains a rectangle that implicitly resizes whenever children are
 * added to or removed from the group.
 *
 * The following contains an example that adds several shapes to a boxed group,
 * then rotates the entire group by 45 degrees counter-clockwise.
 *
 * \image html boxed.png "Boxed group and shapes drawn by example code below"
 *
 * \dontinclude factory.cpp
 * \skip example_boxed
 * \until }
 *
 */

namespace Papyrus {

  /**
   * @author Rick L Vinyard Jr
   */
  class Boxed : public Group {
    public:
      typedef PapyrusSmartPointer<Boxed> pointer;

      Boxed();

      static Boxed::pointer create();

      virtual ~Boxed();

      Rectangle::pointer get_box();

      virtual bool add( Drawable::pointer item, int layer=0, bool drawing_only=true );

      PAPYRUS_CLASS_NAME("Boxed Group");

      PAPYRUS_CLONE_METHOD( Boxed );

    protected:
      Rectangle::pointer m_box;

      /**
       * Reimplements Group's behavior when extents need to be updated.
       * Performs same behavior, but also adjusts rectangle x, y, w, h.
       */
      virtual void rebuild_extents();

      virtual void on_child_added( Drawable::pointer child );

      virtual void on_child_removed( Drawable::pointer child );

    private:

      virtual void on_child_added_proxy( Drawable::pointer child );

      virtual void on_child_removed_proxy( Drawable::pointer child );

  };

}

#endif
