/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSMARKER_H
#define PAPYRUSMARKER_H

#include <papyrus/shape.h>

namespace Papyrus {

  /**
   * Draws an marker head or tail (not the body of the marker) with the tip
   * or tail centered at (x,y).
   *
   * @author Rick L Vinyard Jr <rvinyard@cs.nmsu.edu>
   */
  class Marker : public Shape {
    public:
      typedef enum {
        NONE,
        LINES,
        TRIANGLE,
        DIAMOND,
        HALF_DIAMOND,
        HALF_HEAD_UPPER,
        HALF_HEAD_LOWER,
        SLASHED_CROSS,
        ELLIPSE,
        DOT,
        DOUBLE_TRIANGLE,
        DOT_AND_TRIANGLE,
        BOX,
        CROW_FOOT,
        CROSS,
        ONE_OR_MANY,
        ZERO_OR_MANY,
        ONE_OR_ZERO,
        ONE_EXACTLY,
        CONCAVE,
        ROUND,
        OPEN_ROUND,
        BACKSLASH,
    } STYLE;

      typedef enum {
        LEFT,
        RIGHT
    } FACING;

      typedef PapyrusSmartPointer<Marker> pointer;

      /** Default constructor */
      Marker( STYLE style, double width = 20.0, double height = 20.0, FACING facing = RIGHT );

      /** Creates an marker with the specified style. */
      static Marker::pointer create( STYLE style=NONE, double width = 20.0, double height = 20.0, FACING facing = RIGHT );

      /** Destructor */
      virtual ~Marker();

      /** @return marker style */
      STYLE style() const;

      /** set marker style */
      void set_style( STYLE style );

      /** @return drawing width of marker */
      double width() const;

      /** set drawing width of marker */
      void set_width( double width );

      /** @return drawing height of marker */
      double height() const;

      /** set drawing height of marker */
      void set_height( double height );

      /** get the drawing width and height of the marker */
      void width_height( double& width, double& height ) const;

      /** set the drawing width and height of the marker */
      void set_width_height( double width, double height );

      /** set the line drawing style */
      LineStyle& line_style();

      /**
       * @return the facing direction.
       * Changing the facing direction is more than a simple 180' rotation.
       * It effects a mirror of the marker about the marker's local y-axis.
       */
      FACING facing() const;

      /** set the facing direction */
      void set_facing( FACING facing );

      /** Draws on the provided context */
      virtual void draw_shape( Cairo::RefPtr<Cairo::Context> cairo );

      /** Signal emitted when the style is changed */
      sigc::signal<void> signal_style();

      /** Signal emitted when the width or height is changed */
      sigc::signal<void> signal_size();

      /** Signal emitted when the facing direction is changed */
      sigc::signal<void> signal_facing();

      PAPYRUS_CLASS_NAME("Marker");

      PAPYRUS_CLONE_METHOD( Marker );

		protected:
      STYLE m_style;

      double m_width;

      double m_height;

      FACING m_facing;

      virtual void need_redraw();

      /** Signal emitted when the style is changed */
      sigc::signal<void> m_signal_style;

      /** Signal emitted when the width or height is changed */
      sigc::signal<void> m_signal_size;

      /** Signal emitted when the facing direction is changed */
      sigc::signal<void> m_signal_facing;
  };

}

#endif
