/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSRECTANGLE_H
#define PAPYRUSRECTANGLE_H

#include <papyrus/shape.h>

/**
 * \page guide_rectangle Rectangles
 * A rectangle is drawn with the (x,y) coordinate as the upper left corner, and with a
 * specified width and height.
 *
 * Rotation of a rectangle occurs about a centroid which is defined as the center of the
 * rectangle, or more formally as (x+w/2, y+h/2).
 *
 * The following contains an example that will add a rectangle with a border to a canvas.
 * The blue lines represent the x and y axes, and show through since an alpha value of
 * 0.9 is used.
 *
 * \image html rectangle.png "Rectangle drawn by example code below"
 *
 * \dontinclude factory.cpp
 * \skip example_rectangle
 * \until }
 *
 */

namespace Papyrus {

  /**
   * A rectangle centered on the x,y coordinate.
   *
   * \image html rectangle.png "Rectangle"
   *
   * @author Rick L Vinyard Jr
   */
  class Rectangle : public Shape {
    protected:

      Rectangle( double w,
                 double h,
                 Cairo::RefPtr<Cairo::Pattern> fill
               );

      Rectangle( double w,
                 double h,
                 Cairo::RefPtr<Cairo::Pattern> fill,
                 const LineStyle& outline
               );

    public:
      typedef PapyrusSmartPointer<Rectangle> pointer;

      /** Creates a rectangle of a specified width and height */
      static Rectangle::pointer create( double w = 0.0,
                                        double h = 0.0,
                                        Cairo::RefPtr<Cairo::Pattern> fill = Cairo::RefPtr<Cairo::Pattern>()
                                      );

      /** Creates a rectangle of a specified width and height */
      static Rectangle::pointer create( double w,
                                        double h,
                                        Cairo::RefPtr<Cairo::Pattern> fill,
                                        const LineStyle& outline
                                      );

      virtual ~Rectangle();

      /** Returns the width of the rectangle */
      double width();

      /** Returns the height of the rectangle */
      double height();

      /** Fills parameters w and h with the width and height of the rectangle */
      void width_height( double& w, double& h );

      /** Sets the width of the rectangle to w */
      void set_width( double w );

      /** Sets the height of the rectangle to h */
      void set_height( double h );

      /** Sets the width to w and height to h */
      void set_width_height( double w, double h );

      /**
       * Sets the center of the rectangle to (x,y) and also sets the width
       * and height of the rectangle.
       */
      void set_xywh( double x, double y, double w, double h );

      /** Draws on the provided context */
      virtual void draw_shape( Cairo::RefPtr<Cairo::Context> cairo );

      /** signal emitted when the size of the rectangle is changed */
      sigc::signal<void> signal_size();

      PAPYRUS_CLASS_NAME("Rectangle");

      PAPYRUS_CLONE_METHOD( Rectangle );

  protected:

      double m_width;

      double m_height;

      virtual void need_redraw();

      sigc::signal<void> m_signal_size;

  };

}

#endif
