/***************************************************************************
 *   Copyright (C) 2004 by Rick L. Vinyard, Jr.                            *
 *   rvinyard@cs.nmsu.edu                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU Lesser General Public License as        *
 *   published by the Free Software Foundation version 2.1.                *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU Lesser General Public      *
 *   License along with this library; if not, write to the                 *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
 ***************************************************************************/
#ifndef PAPYRUSRENDERABLE_H
#define PAPYRUSRENDERABLE_H

#include <papyrus/smart_pointer.h>

#include <papyrus/object.h>
#include <cairomm/cairomm.h>
#include <iostream>

#define PAPYRUS_CLASS_NAME( StrName )    \
virtual const std::string& class_name()  \
{                                        \
  static const std::string cn(StrName);  \
  return cn;                             \
}

namespace Papyrus
{

  /**
   * Papyrus::Renderable is a common base for all classes capable of rendering themselves
   * into a cairomm context.
   *
   * Renderable provides the pure virtual \c render() method, which takes a
   * cairomm context as its only parameter. Descendants of Papyrus::Renderable
   * provide their own implementations of \c render() that allow them to be
   * rendered into the supplied cairomm context.
   *
   * The \c render() method makes it possible to render a hierarchy (or hierarchies)
   * of Renderables into a cairomm context employing a visitor pattern.
   *
   * <h2>Inheriting from Papyrus::Renderable</h2>
   *
   * <h3>When:</h3>
   * Inherit from Papyrus::Renderable when you want your object to be capable of being
   * rendered into a cairo context through a common interface.
   *
   * <h3>Tips:</h3>
   * Don't forget to provide your own implementation of \c render() since it's
   * a pure virtual method.
   *
   * @author Rick L Vinyard Jr
   */
  class Renderable: public Object
  {
    public:
      typedef PapyrusSmartPointer<Renderable> pointer;

      Renderable() : Object()
      { }

      virtual ~Renderable()
      { }

      virtual void render( Cairo::RefPtr<Cairo::Context> cairo ) = 0;

      PAPYRUS_CLASS_NAME("Renderable");

  };

};

#endif
